module Phylo.Infos where

import Prelude

import DOM.Simple.Console (log, log2)
import Data.Array (length)
import Data.Int (toNumber, round)
import Data.Maybe (Maybe(..), isJust, maybe)
import Data.Nullable (null, toMaybe)
import Effect (Effect)
import Effect.Aff (Aff, killFiber, launchAff, launchAff_)
import Effect.Class (liftEffect)
import Effect.Exception (error)
import FFI.Simple (delay)
import Graphics.D3.Base (D3Eff)
import Graphics.D3.Contour as Contour
import Graphics.D3.Selection as Selection
import Phylo.TypesExample as PTE
import Reactix as R
import Reactix.DOM.HTML as H
import Toestand as T

type Props = ()

infoCorpusR :: R.Element
infoCorpusR = infoCorpus {}

infoPhyloR :: R.Element
infoPhyloR = infoPhylo {}

--infoPhyloCpt :: R.Component Props
--infoPhyloCpt = R.hooksComponent "infoPhylo" cpt where
--  cpt {  } _children = do
--    pure $ H.div {className:"phyloInfos"} [H.text "Here lies Phylo-info",
--    H.div {className:"phyloInfos-info"}[H.text "0"]]

infoPhyloCpt :: R.Component Props
infoPhyloCpt = R.hooksComponent "infoCorpus" cpt where
  cpt {  } _children = do
    fetched <- T.useBox Nothing

    R.useEffect' $ do
      launchAff_ $ do        
        mr <- PTE.fetchPhyloJSON
        liftEffect $ do
          case mr of
            Nothing -> log "Nothing"
            Just r -> T.write_ (Just r) fetched
    pure $ infoPhyloLoaded { fetched } []

--infoCorpusCpt :: R.Component InfoCorpus
--infoCorpusCpt = R.hooksComponent "infoCorpus" cpt where
--  cpt {  } _children = do
--    pure $ H.div {className:"phyloCorpus"} [H.text "Here lies Corpus-info",
--    H.div {className:"phyloCorpus-info"}[H.text $ "Corpus Info here"]

infoCorpus :: Record Props -> R.Element 
infoCorpus props = R.createElement infoCorpusCpt props []

infoPhylo :: Record Props -> R.Element
infoPhylo props = R.createElement infoPhyloCpt props []

type InfoCorpusLoaded =
  ( fetched :: T.Box (Maybe PTE.PhyloJSON) )

infoCorpusCpt :: R.Component Props
infoCorpusCpt = R.hooksComponent "infoCorpus" cpt where
  cpt {  } _children = do
    fetched <- T.useBox Nothing

    R.useEffect' $ do
      launchAff_ $ do        
        mr <- PTE.fetchPhyloJSON
        liftEffect $ do
          case mr of
            Nothing -> log "Nothing"
            Just r -> T.write_ (Just r) fetched
    pure $ infoCorpusLoaded { fetched } []

infoCorpusLoaded :: Record InfoCorpusLoaded -> Array R.Element -> R.Element
infoCorpusLoaded = R.createElement infoCorpusLoadedCpt

infoPhyloLoaded :: Record InfoCorpusLoaded -> Array R.Element -> R.Element
infoPhyloLoaded = R.createElement infoPhyloLoadedCpt

infoCorpusLoadedCpt :: R.Component InfoCorpusLoaded
infoCorpusLoadedCpt = R.hooksComponent "infoCorpusLoaded" cpt where
  cpt { fetched } _ = do
    fetched' <- T.useLive T.unequal fetched

    case fetched' of
      Nothing -> pure $ H.div {}[]
      Just mr -> pure $ H.div {className:"phyloCorpus"} 
        [H.text "Corpus",
         H.div {className:"phyloInfos-info"}[H.span {}[H.text $ mr.phyloDocs <> " docs"],
                                              H.span {}[H.text $ mr.phyloFoundations <> " foundations"],
                                              H.span {}[H.text $ mr.phyloPeriods <> " periods"]]]

infoPhyloLoadedCpt :: R.Component InfoCorpusLoaded
infoPhyloLoadedCpt = R.hooksComponent "infoCorpusLoaded" cpt where
  cpt { fetched } _ = do
    fetched' <- T.useLive T.unequal fetched

    case fetched' of
      Nothing -> pure $ H.div {}[]
      Just mr -> pure $ H.div {className:"phyloInfos"} 
        [H.text "Phylo",
         H.div {className:"phyloCorpus-info"}[H.span {}[H.text $ mr.phyloTerms <> " Termes"],
                                              H.span {}[H.text $ mr.phyloBranches <> " Branches"],
                                              H.span {}[H.text $ mr.phyloGroups <> " Groupes"]]]
--getPhyloName = 
--  launchAff_ $ do
--    mr <- PTE.fetchPhyloJSON
--    liftEffect $ do
--      case mr of
--        Nothing -> log "Nothing"
--        Just r  -> log2 "r" r.name
--      pure $ maybe mr.name