"use strict";

exports.drawIsoline = function(phyloBranches){
    return function (el){
    /* ** draw the isoline ** */
        var branches = phyloBranches.value0.map(function (b) {
            return {
              x1: b.branch_x,
              y: b.branch_y,
              x2: parseFloat(b.pos.split(",")[0]),
              label: b.label,
              bId: parseInt(b.bId),
              gvid: parseInt(b._gvid)
            };
          });

        d3.select(el).style("background", "#EBE4DD");
        var div0 = d3.select(el).node().getBoundingClientRect(),
          m0 = { t: 5, r: 5, b: 5, l: 5 },
          w0 = div0.width,
          h0 = div0.height;
      
        var svg0 = d3
          .select(el)
          .append("svg")
          .attr("width", w0)
          .attr("height", h0)
          .append("g");
      
        var xScale0 = d3
            .scaleLinear()
            .domain([0, Math.max(...branches.map((b) => b.x1))])
            .range([2 * m0.l, w0 - 2 * m0.l]),
          yScale0 = d3
            .scaleLinear()
            .domain(d3.extent(branches, (b) => b.y))
            .nice()
            .range([2 * m0.t, h0 - 2 * m0.t]);
      
        var density = d3
          .contourDensity()
          .x(function (b) {
            return xScale0(b.x1);
          })
          .y(function (b) {
            return yScale0(b.y);
          })
          .size([w0, h0])
          .thresholds(Math.round(branches.length / 2))(branches);
      
        /* shadows and lights */
      
        svg0
          .append("g")
          .selectAll("circle")
          .data(branches)
          .enter()
          .append("circle")
          .attr("cx", (b) => xScale0(b.x1))
          .attr("cy", (b) => yScale0(b.y))
          .attr("r", "55")
          .attr("id", (b) => "peak-shadow" + b.bId)
          .attr("visibility", "visible")
          .style("fill", "#f5eee6");
      
        svg0
          .selectAll("path")
          .data(density)
          .enter()
          .append("path")
          .attr("d", d3.geoPath())
          .attr("fill", "none")
          .attr("stroke", "#74B5FF")
          .attr("stroke-width", (d, i) => (i % 2 ? 0.25 : 1))
          .attr("stroke-linejoin", "round");
      
        var label = d3
          .select(el)
          .append("div")
          .attr("class", "peak-label");
      
        svg0
          .append("g")
          .selectAll("text")
          .data(branches)
          .enter()
          .append("text")
          .attr("x", (b) => xScale0(b.x1))
          .attr("y", (b) => yScale0(b.y) + 4)
          .attr("class", "peak")
          .attr("id", (b) => "peak-" + b.bId)
          .style("fill", "#0d1824")
          .attr("visibility", "visible")
          .text("▲");
        };
  };