module Test where

import Prelude

import Affjax as AX
import Affjax.ResponseFormat as ResponseFormat
import DOM.Simple.Console (log, log2)
import Data.Either (Either(..))
import Data.HTTP.Method (Method(..))
import Data.Maybe (Maybe(..), fromMaybe, maybe)
import Effect (Effect)
import Effect.Aff (Aff, launchAff, launchAff_)
import Effect.Class (liftEffect)
import Simple.JSON as JSON

type PhyloJSON = 
  { name              :: String
  , phyloDocs         :: String
  , phyloFoundations  :: String
  , phyloPeriods      :: String
  , phyloTerms        :: String
  , phyloGroups       :: String
  , phyloBranches     :: String
  , objects           :: Array PhyloObject
  , edges             :: Array PhyloEdge
  }

data PhyloObject = Branch  
                { _gvid :: Int
                , bId :: String
                , nodeType :: String
                } | Group   
                { _gvid :: Int
                , bId :: String
                , nodeType :: String
                } | Period  
                { _gvid :: Int
                , nodeType :: String
                } | Default 
                {
                  _gvid :: Int
                }

newtype DBranch = DBranch
    { _gvid    :: Int
    , bId      :: String
    , nodeType :: String
    }
newtype DPeriod = DPeriod
    { _gvid    :: Int
    , bId      :: String
    , nodeType :: String
    }
newtype DGroup = DGroup
    { _gvid    :: Int
    , bId      :: String
    , nodeType :: String
    }

    
instance JSON.ReadForeign PhyloObject where
  readImpl f = do
    { _gvid, nodeType: mNodeType } :: { _gvid :: Int, nodeType :: Maybe String } <- JSON.readImpl f
    case mNodeType of
      Nothing -> pure $ Default { _gvid }
      Just nodeType -> do
        { bId: mBId } :: { bId :: Maybe String } <- JSON.readImpl f
        case mBId of
          Nothing -> pure $ Default { _gvid }
          Just bId ->
            case nodeType of
              "branch" -> pure $ Branch { _gvid, bId, nodeType }
              "group" -> pure $ Group { _gvid, bId, nodeType }
              "period" -> pure $ Period { _gvid, nodeType }
              _ -> pure $ Default { _gvid }

-- :: Array PhyloObject -> Array DBranch
-- x = x

data PhyloEdge =  Link  
                { _gvid :: Int
                , bId :: String
                , edgeType :: String
                } | AncestorLink   
                { _gvid :: Int
                , bId :: String
                , edgeType :: String
                } | BranchLink  
                { _gvid :: Int
                , bId :: String
                , edgeType :: String
                } | DefaultLink
                {
                  _gvid :: Int
                }

newtype DLink = DLink
    { _gvid    :: Int
    , bId      :: String
    , edgeType :: String
    }
newtype DAncestorLink = DAncestorLink
    { _gvid    :: Int
    , bId      :: String
    , edgeType :: String
    }
newtype DBranchLink = DBranchLink
    { _gvid    :: Int
    , bId      :: String
    , edgeType :: String
    }
